package com.eii.tooltray;

import java.util.*;

/**
* A JNI wrapper for desktop indicators (also called tray icons and taskbar icons).
* <br><br>
*
* The actual implementation is platform specific. Currently only Win32 is supported.
* For unsupported platforms, the desktop indicator doesn't do anything, and is
* otherwise harmless. Likewise, some platforms may not support the tooltip
* feature.
* <br><br>
*
* IMPORTANT!
* <br><br>
*
* Relying on this feature will make your applet or application platform-
* dependant. If possible, treat is as an "added benefit", not a required feature.
**/

public class DesktopIndicator
{
	///////////////////////////////////////////////////////////////////////////////////////
	// Private
	//

	private int image;
	private String tooltip;
	private Vector listeners = new Vector();
	private int handler = 0;

/**
 * Insert the method's description here.
 * Creation date: (6/24/99 2:49:49 PM)
 */
public DesktopIndicator() {
	
}
	/**
	* Creates a desktop indicator, initially hidden.
	* <br><br>
	*
	* The image handle is the return value of a previous call to loadImage.
	**/
	public DesktopIndicator( int image, String tooltip )
	{
		this.image = image;
		this.tooltip = tooltip;
	}
	/**
	* Adds a listener for clicks.
	**/	
	public void addDesktopIndicatorListener( DesktopIndicatorListener listener )
	{
		listeners.addElement( listener );
	}
	/**
	* Makes sure that the desktop indicator is hidden.
	**/
	protected void finalize()
	{
		nativeDisable();
	}
	/**
	* Notifies all listeners that the desktop indicator was clicked.
	**/	
	public void fireClicked()
	{
		Vector listenersClone = (Vector) listeners.clone();
		DesktopIndicatorListener listener;
		for( Enumeration e = listenersClone.elements(); e.hasMoreElements(); )
		{
			listener = (DesktopIndicatorListener) e.nextElement();
			listener.onDesktopIndicatorClicked( this );				
		}
	}
	/**
	* Frees memory used by a image previously loaded with loadImage.
	* <br><br>
	*
	* The memory is freed in any case when the VM is closed.
	**/
	public static void freeImage( int image )
	{
		try
		{
			nativeFreeImage( image );
		}
		catch( UnsatisfiedLinkError x )
		{
		}
	}
	/**
	* Hides the desktop indicator.
	**/
	public void hide()
	{
		try
		{
			nativeHide();
		}
		catch( UnsatisfiedLinkError e )
		{
		}
	}
	/**
	* Loads the JNI library, if available.
	* <br><br>
	*
	* Must be called before images are loaded and instances are created.
	**/
	public static boolean initialize()
	{
		// Load JNI library
		try
		{
			System.loadLibrary( "DesktopIndicator" );
		}
		catch( UnsatisfiedLinkError x )
		{
			return false;
		}
		
		return true;
	}
	/**
	* Loads an image file to memory.
	* <br><br>
	*
	* The image is loaded from the filename parameter, while the tooltip is
	* supplied in the tooltip parameter. The image file must be in a format
	* recognized by the native platform (for example, a .ico file for Win32).
	* <br><br>
	*
	* A return value of -1 indicates failure. Otherwise, it is the handle of
	* the image to be used for desktop indicators.
	**/
	public static int loadImage( String filename )
	{
		try
		{
			return nativeLoadImage( filename );
		}
		catch( UnsatisfiedLinkError x )
		{
			return -1;
		}
	}
	/**
	* Loads an image file to memory.
	* <br><br>
	*
	* The image is loaded from the filename parameter, while the tooltip is
	* supplied in the tooltip parameter. The image file must be in a format
	* recognized by the native platform (for example, a .ico file for Win32).
	* <br><br>
	*
	* A return value of -1 indicates failure. Otherwise, it is the handle of
	* the image to be used for desktop indicators.
	**/
	public static int loadImageFromResource( int inResourceNum )
	{
		try
		{
			return nativeLoadImageFromResource( inResourceNum );
		}
		catch( UnsatisfiedLinkError x )
		{
			return -1;
		}
	}
	private synchronized native void nativeDisable() throws UnsatisfiedLinkError;
	private synchronized native void nativeEnable( int image, String tooltip ) throws UnsatisfiedLinkError;
	private synchronized static native void nativeFreeImage( int image ) throws UnsatisfiedLinkError;
	private synchronized static native int nativeLoadImage( String filename ) throws UnsatisfiedLinkError;
	private synchronized static native int nativeLoadImageFromResource( int inResource ) throws UnsatisfiedLinkError;
	private synchronized native void nativeHide() throws UnsatisfiedLinkError;
	
	/**
	* Removes a listener for clicks.
	**/	
	public void removeDesktopIndicatorListener( DesktopIndicatorListener listener )
	{
		listeners.removeElement( listener );
	}
	/**
	* Enables the desktop indicator.
	* <br><br>
	*
	* If the indicator is already active, it is "refreshed" with the filename and tooltip.
	**/
	public void show()
	{
		try
		{
			nativeEnable( image, tooltip );
		}
		catch( UnsatisfiedLinkError x )
		{
		}
	}
	/**
	* Updates the desktop indicator with an image and tooltip.
	**/
	public void update( int image, String tooltip )
	{
		this.image = image;
		this.tooltip = tooltip;
		
		try
		{
			nativeEnable( image, tooltip );
		}
		catch( UnsatisfiedLinkError x )
		{
		}
	}
}
